"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const backo2_1 = __importDefault(require("backo2"));
const eventemitter3_1 = require("eventemitter3");
const interpreter_1 = require("xstate/lib/interpreter");
const protocol_1 = require("../protocol");
const machine_1 = require("./machine");
const operationProcessor_1 = __importDefault(require("./operationProcessor"));
const _global = typeof global !== 'undefined'
    ? global
    : typeof window !== 'undefined'
        ? window
        : {};
const NativeWebSocket = _global.WebSocket || _global.MozWebSocket;
class Client {
    constructor({ uri, options: { lazy = false, operationTimeout = Infinity, reconnect = false, reconnectAttempts = Infinity, } = {}, webSockImpl = NativeWebSocket, }) {
        this.disconnect = () => {
            this.machine.send('DISCONNECT');
        };
        this.on = (event, listener) => {
            this.ee.on(event, listener);
            return () => this.ee.off(event, listener);
        };
        this.onConnecting = (listener) => this.on('connecting', listener);
        this.onConnected = (listener) => this.on('connected', listener);
        this.onDisconnected = (listener) => this.on('disconnected', listener);
        this.onError = (listener) => this.on('error', listener);
        this.onMessage = (listener) => this.on('message', listener);
        this.onReconnecting = (listener) => this.on('reconnecting', listener);
        this.onReconnected = (listener) => this.on('reconnected', listener);
        this.handleMessage = (event) => {
            try {
                const message = JSON.parse(event.data);
                switch (message.type) {
                    case protocol_1.SERVER_EVENT_TYPES.GQL_OP_RESULT: {
                        this.operationProcessor.processOperationResult(message);
                        break;
                    }
                    case protocol_1.SERVER_EVENT_TYPES.GQL_CONNECTED: {
                        // connected
                    }
                    case protocol_1.SERVER_EVENT_TYPES.GQL_ERROR: {
                        // error
                    }
                    case protocol_1.SERVER_EVENT_TYPES.GQL_SUBSCRIBED: {
                        // subcribed
                        break;
                    }
                    default: {
                        throw new Error('Unknown message');
                    }
                }
                this.ee.emit('message', message);
            }
            catch (e) {
                this.ee.emit('error', e);
                throw e;
            }
        };
        const backoff = new backo2_1.default({ jitter: 0.5 });
        if (webSockImpl == null) {
            throw new Error('Not native WebSocket implementation detected, please provide an implementation');
        }
        this.lazy = lazy;
        this.ee = new eventemitter3_1.EventEmitter();
        this.operationProcessor = new operationProcessor_1.default({ operationTimeout });
        this.machine = interpreter_1.interpret(machine_1.clientMachine.withContext({
            backoff,
            reconnect,
            reconnectAttempts,
            uri,
            handleMessage: this.handleMessage,
            operationProcessor: this.operationProcessor,
            webSockImpl: webSockImpl,
        })).start();
        this.machine.onTransition(state => {
            this.ee.emit(state.value, state.event.data);
        });
        if (!this.lazy) {
            this.machine.send('CONNECT');
        }
    }
    request(operation) {
        if (this.lazy) {
            this.machine.send('CONNECT'); // if client is already connected, this won't do anything
        }
        return this.operationProcessor.execute(operation);
    }
    get status() {
        return this.machine.state.value;
    }
}
exports.Client = Client;
exports.default = Client;
