"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const graphql_1 = require("graphql");
/**
 * Execute methods executes graphql operations
 *
 * In case of mutation/query it returns ExecutionResult
 * In case of subscriptions it returns AsyncIterator of ExecutionResults (only if useSubscriptions is true)
 *
 * @param param0
 */
async function execute({ connection, connectionManager, context, operation, pubSub, rootValue, schema, subscriptionManager, registerSubscriptions = true, useSubscriptions = false, }) {
    // extract query from operation (parse if is string);
    const document = typeof operation.query !== 'string'
        ? operation.query
        : graphql_1.parse(operation.query);
    // validate document
    const validationErrors = graphql_1.validate(schema, document, graphql_1.specifiedRules);
    if (validationErrors.length > 0) {
        return {
            errors: validationErrors,
        };
    }
    // this is internal context that should not be used by a user in resolvers
    // this is only added to provide access for PubSub to get connection managers and other
    // internal stuff
    const $$internal = {
        connection,
        connectionManager,
        operation,
        pubSub,
        registerSubscriptions,
        subscriptionManager,
    };
    // instantiate context
    const contextValue = typeof context === 'function' ? await context() : context;
    // detect operation type
    const operationAST = graphql_1.getOperationAST(document, operation.operationName || '');
    if (useSubscriptions) {
        if (operationAST.operation === 'subscription') {
            return graphql_1.subscribe({
                document,
                rootValue,
                schema,
                contextValue: Object.assign({}, contextValue, { $$internal }),
                operationName: operation.operationName,
                variableValues: operation.variables,
            });
        }
    }
    else if (!useSubscriptions && operationAST.operation === 'subscription') {
        throw new Error('Cannot subscribe using HTTP');
    }
    return graphql_1.execute({
        document,
        rootValue,
        schema,
        contextValue: Object.assign({}, contextValue, { $$internal }),
        operationName: operation.operationName,
        variableValues: operation.variables,
    });
}
exports.execute = execute;
exports.default = execute;
